CREATE TABLE cities (
    id bigint NOT NULL,
    regionid bigint DEFAULT 0,
    loc_region_id bigint DEFAULT 0,
    name character varying(255) DEFAULT NULL::character varying,
    name_genitive character varying(255),
    name_prepositional character varying(255) DEFAULT NULL::character varying,
    name_ukr character varying(255) DEFAULT NULL::character varying,
    name_ukr_genetive character varying(255),
    name_ukr_prepositional character varying(255) DEFAULT NULL::character varying,
    translit character varying(255) DEFAULT NULL::character varying,
    mainflag smallint DEFAULT 0 NOT NULL,
    importance smallint DEFAULT 0,
    size smallint DEFAULT 1,
    population numeric(7,0),
    map boolean DEFAULT false NOT NULL,
    domain character varying(255) DEFAULT NULL::character varying,
    renta_item_count bigint DEFAULT 0 NOT NULL,
    class_flat_count integer[] DEFAULT ARRAY[0, 0, 0, 0],
    class_room_count integer[] DEFAULT ARRAY[0, 0, 0, 0],
    class_house_count integer[] DEFAULT ARRAY[0, 0, 0, 0],
    class_area_count integer[] DEFAULT ARRAY[0, 0, 0, 0],
    class_commercial_count integer[] DEFAULT ARRAY[0, 0, 0, 0],
    pos integer DEFAULT 0,
    user_id bigint DEFAULT 0 NOT NULL,
    mod_user_id bigint DEFAULT 0 NOT NULL,
    name_vector tsvector,
    name_lower character varying(255),
    name_ukr_lower character varying(255),
    created timestamp without time zone,
    modified timestamp without time zone,
    tel_code character varying(8)
);