ALTER TABLE teste
	ADD COLUMN IF NOT EXISTS id integer NOT NULL;

ALTER TABLE teste
	ALTER COLUMN id ADD GENERATED BY DEFAULT AS IDENTITY (
    SEQUENCE NAME public.teste_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1
);