
ALTER TABLE testtable
	ADD COLUMN IF NOT EXISTS col3 boolean DEFAULT false NOT NULL,
	ADD COLUMN IF NOT EXISTS col4 character(10) DEFAULT '' NOT NULL,
	ADD COLUMN IF NOT EXISTS col5 text DEFAULT '' NOT NULL,
	ALTER COLUMN col1 SET DEFAULT 0,
	ALTER COLUMN col1 SET NOT NULL;

ALTER TABLE testtable
	ALTER COLUMN col3 DROP DEFAULT,
	ALTER COLUMN col4 DROP DEFAULT,
	ALTER COLUMN col5 DROP DEFAULT,
	ALTER COLUMN col1 DROP DEFAULT;
